@@ -21,7 +21,6 @@
 from datetime import datetime
 from fractions import Fraction
 
-from gi.repository import GObject
 from ..overrides import override
 from ..module import get_introspection_module
 
@@ -78,6 +77,9 @@ class Metadata(GExiv2.Metadata):
     
     def get(self, key, default=None):
         return self.get_tag_string(key) if self.has_tag(key) else default
+
+    def get_raw(self, key):
+        return self.get_tag_raw(key).get_data()
     
     def __iter__(self):
         return iter(self.get_tags())
@@ -102,8 +104,8 @@ class Metadata(GExiv2.Metadata):
     
     __setitem__ = GExiv2.Metadata.set_tag_string
 
+
 __all__.append('Metadata')
 
 if not GExiv2.initialize():
     raise RuntimeError("GExiv2 couldn't be initialized")
-
